@@ -53,6 +53,7 @@
 #include <link-grammar/error.h>
 
 #include "command-line.h"
+#include "expand.h"
 
 #define MAX_INPUT 1024
 #define DISPLAY_MAX 1024
@@ -427,7 +428,7 @@ static Label strip_off_label(char * input_string)
 
 static void setup_panic_parse_options(Parse_Options opts)
 {
-	parse_options_set_disjunct_cost(opts, 3);
+	parse_options_set_disjunct_costf(opts, 3.0f);
 	parse_options_set_min_null_count(opts, 1);
 	parse_options_set_max_null_count(opts, MAX_SENTENCE);
 	parse_options_set_max_parse_time(opts, 60);
@@ -552,6 +553,9 @@ int main(int argc, char * argv[])
 
 	verbosity = parse_options_get_verbosity(opts);
 
+	prt_error("Info: Version %s. Enter \"!help\" for help.\n",
+		linkgrammar_get_version());
+
 	/* Main input loop */
 	while (1)
 	{
@@ -586,11 +590,13 @@ int main(int argc, char * argv[])
 		}
 
 		if (special_command(input_string, dict)) continue;
-		if (parse_options_get_echo_on(opts)) {
+		if (parse_options_get_echo_on(opts))
+		{
 			printf("%s", input_string);
 		}
 
-		if (parse_options_get_batch_mode(opts)) {
+		if (parse_options_get_batch_mode(opts))
+		{
 			label = strip_off_label(input_string);
 		}
 
@@ -598,8 +604,10 @@ int main(int argc, char * argv[])
 
 		if (sent == NULL) continue;
 
-		if (sentence_length(sent) > parse_options_get_max_sentence_length(opts)) {
-			if (verbosity > 0) {
+		if (sentence_length(sent) > parse_options_get_max_sentence_length(opts))
+		{
+			if (verbosity > 0)
+			{
 				fprintf(stdout,
 				       "Sentence length (%d words) exceeds maximum allowable (%d words)\n",
 					sentence_length(sent), parse_options_get_max_sentence_length(opts));
@@ -609,7 +617,7 @@ int main(int argc, char * argv[])
 		}
 
 		/* First parse with cost 0 or 1 and no null links */
-		parse_options_set_disjunct_cost(opts, 2);
+		parse_options_set_disjunct_costf(opts, 2.0f);
 		parse_options_set_min_null_count(opts, 0);
 		parse_options_set_max_null_count(opts, 0);
 		parse_options_reset_resources(opts);
@@ -617,17 +625,36 @@ int main(int argc, char * argv[])
 		num_linkages = sentence_parse(sent, opts);
 		if (num_linkages < 0) continue;
 
+		/* Try using a larger list of disjuncts */
+		if ((num_linkages == 0) && parse_options_get_use_cluster_disjuncts(opts))
+		{
+			int expanded;
+			if (verbosity > 0) fprintf(stdout, "No standard linkages, expanding disjunct set.\n");
+			parse_options_set_disjunct_costf(opts, 2.9f);
+			expanded = lg_expand_disjunct_list(sent);
+			if (expanded)
+			{
+				num_linkages = sentence_parse(sent, opts);
+			}
+			if (0 < num_linkages) printf("Got One !!!!!!!!!!!!!!!!!\n");
+		}
+
 		/* Now parse with null links */
-		if ((num_linkages == 0) && (!parse_options_get_batch_mode(opts))) {
+		if ((num_linkages == 0) && (!parse_options_get_batch_mode(opts)))
+		{
 			if (verbosity > 0) fprintf(stdout, "No complete linkages found.\n");
-			if (parse_options_get_allow_null(opts)) {
+
+			if (parse_options_get_allow_null(opts))
+			{
+				/* XXX should use expanded disjunct list here too */
 				parse_options_set_min_null_count(opts, 1);
 				parse_options_set_max_null_count(opts, sentence_length(sent));
 				num_linkages = sentence_parse(sent, opts);
 			}
 		}
 
-		if (parse_options_timer_expired(opts)) {
+		if (parse_options_timer_expired(opts))
+		{
 			if (verbosity > 0) fprintf(stdout, "Timer is expired!\n");
 		}
 		if (parse_options_memory_exhausted(opts)) {
@@ -636,7 +663,8 @@ int main(int argc, char * argv[])
 
 		if ((num_linkages == 0) &&
 			parse_options_resources_exhausted(opts) &&
-			parse_options_get_panic_mode(opts)) {
+			parse_options_get_panic_mode(opts))
+		{
 			/* print_total_time(opts); */
 			if (verbosity > 0) fprintf(stdout, "Entering \"panic\" mode...\n");
 			parse_options_reset_resources(panic_parse_opts);
@@ -649,10 +677,12 @@ int main(int argc, char * argv[])
 
 		/* print_total_time(opts); */
 
-		if (parse_options_get_batch_mode(opts)) {
+		if (parse_options_get_batch_mode(opts))
+		{
 			batch_process_some_linkages(label, sent, opts);
 		}
-		else {
+		else
+		{
 			int c = process_some_linkages(sent, opts);
 			if (c == EOF) break;
 		}
@@ -660,7 +690,8 @@ int main(int argc, char * argv[])
 		sentence_delete(sent);
 	}
 
-	if (parse_options_get_batch_mode(opts)) {
+	if (parse_options_get_batch_mode(opts))
+	{
 		/* print_time(opts, "Total"); */
 		fprintf(stderr,
 				"%d error%s.\n", batch_errors, (batch_errors==1) ? "" : "s");
