@@ -17,11 +17,17 @@
 #endif
 #include <limits.h>
 
-#include <link-grammar/api.h>
+#include "build-disjuncts.h"
 #include "error.h"
+#include "externs.h"
+#include "read-dict.h"
 #include "regex-morph.h"
 #include "spellcheck-hun.h"
+#include "string-set.h"
+#include "structures.h"
+#include "tokenize.h"
 #include "utilities.h"
+#include "word-utils.h"
 
 #define MAX_STRIP 10
 
@@ -30,6 +36,7 @@
 /*static char * strip_left[] = {"(", "$", "``", NULL}; */
 /*static char * strip_right[] = {")", "%", ",", ".", ":", ";", "?", "!", "''", "'", "'s", NULL};*/
 
+#define ENTITY_MARKER   "<marker-entity>"
 #define COMMON_ENTITY_MARKER   "<marker-common-entity>"
 
 /**
@@ -45,6 +52,13 @@ static int is_common_entity(Dictionary dict, const char * str)
 	return 0;
 }
 
+static int is_entity(Dictionary dict, const char * str)
+{
+	if (word_contains(dict, str, ENTITY_MARKER) == 1)
+		return 1;
+	return 0;
+}
+
 /**
  * Is the word entirely composed of single-letter abreviations
  * (followed by a period)?
@@ -771,7 +785,7 @@ static int special_string(Sentence sent, int i, const char * s)
 	X_node * e;
 	if (boolean_dictionary_lookup(sent->dict, s))
 	{
-		sent->word[i].x = build_word_expressions(sent, s);
+		sent->word[i].x = build_word_expressions(sent->dict, s);
 		for (e = sent->word[i].x; e != NULL; e = e->next)
 		{
 			e->string = sent->word[i].string;
@@ -801,7 +815,7 @@ static void tag_regex_string(Sentence sent, int i, const char * type)
 	char str[MAX_WORD+1];
 	char * t;
 	X_node * e;
-	sent->word[i].x = build_word_expressions(sent, type);
+	sent->word[i].x = build_word_expressions(sent->dict, type);
 	for (e = sent->word[i].x; e != NULL; e = e->next)
 	{
 		t = strchr(e->string, '.');
@@ -831,7 +845,7 @@ static int guessed_string(Sentence sent, int i, const char * s, const char * typ
 	char str[MAX_WORD+1];
 	if (boolean_dictionary_lookup(sent->dict, type))
 	{
-		sent->word[i].x = build_word_expressions(sent, type);
+		sent->word[i].x = build_word_expressions(sent->dict, type);
 		e = sent->word[i].x;
 		if(is_s_word(s))
 		{
@@ -896,7 +910,7 @@ static void handle_unknown_word(Sentence sent, int i, char * s)
 	X_node *d;
 	char str[MAX_WORD+1];
 
-	sent->word[i].x = build_word_expressions(sent, UNKNOWN_WORD);
+	sent->word[i].x = build_word_expressions(sent->dict, UNKNOWN_WORD);
 	if (sent->word[i].x == NULL)
 		assert(FALSE, "UNKNOWN_WORD should have been there");
 
@@ -951,7 +965,7 @@ static void guess_misspelled_word(Sentence sent, int i, char * s)
 	{
 		if (boolean_dictionary_lookup(sent->dict, alternates[j]))
 		{
-			X_node *x = build_word_expressions(sent, alternates[j]);
+			X_node *x = build_word_expressions(sent->dict, alternates[j]);
 			head = catenate_X_nodes(x, head);
 		}
 	}
@@ -1023,7 +1037,7 @@ int build_sentence_expressions(Sentence sent, Parse_Options opts)
 		s = sent->word[i].string;
 		if (boolean_dictionary_lookup(sent->dict, s))
 		{
-			sent->word[i].x = build_word_expressions(sent, s);
+			sent->word[i].x = build_word_expressions(sent->dict, s);
 		}
 #if DONT_USE_REGEX_GUESSING
 		else if (is_utf8_upper(s) && is_s_word(s) && dict->pl_capitalized_word_defined) 
@@ -1123,6 +1137,12 @@ int build_sentence_expressions(Sentence sent, Parse_Options opts)
 		 * to be capitalized, while preventing an upper-case "She" being
 		 * used as a proper name in "She declared bankruptcy".
 		 *
+		 * Arghh. This is still messed up. The capitalized-regex runs
+		 * too early, I think. We need to *add* Sue.f (female name Sue)
+		 * even though sue.v (the verb "to sue") is in the dict. So 
+		 * test for capitalized entity names. Glurg. Too much complexity
+		 * here, it seems to me.
+		 *
 		 * This is actually a great example of a combo of an algorithm
 		 * together with a list of words used to determine grammatical
 		 * function.
@@ -1135,16 +1155,17 @@ int build_sentence_expressions(Sentence sent, Parse_Options opts)
 
 			if (boolean_dictionary_lookup(sent->dict, lc))
 			{
-				if (is_common_entity(sent->dict,lc))
+				if (is_entity(sent->dict,s) ||
+				    is_common_entity(sent->dict,lc))
 				{
-					e = build_word_expressions(sent, lc);
+					e = build_word_expressions(sent->dict, lc);
 					sent->word[i].x =
 						catenate_X_nodes(sent->word[i].x, e);
 				}
 				else
 				{
 					safe_strcpy(s, lc, MAX_WORD);
-					e = build_word_expressions(sent, s);
+					e = build_word_expressions(sent->dict, s);
 					free_X_nodes(sent->word[i].x);
 					sent->word[i].x = e;
 				}
