@@ -14,10 +14,180 @@
  * Miscellaneous utilities for dealing with word types.
  */
 
-#include <link-grammar/api.h>
-#include "word-utils.h"
+#include <math.h>
 #include <stdio.h>
 
+#include <link-grammar/api.h>
+#include "disjunct-utils.h"
+#include "word-utils.h"
+
+/* ======================================================== */
+/* Exp utilities ... */
+
+void free_E_list(E_list *);
+void free_Exp(Exp * e)
+{
+	if (e->type != CONNECTOR_type) {
+		free_E_list(e->u.l);
+	}
+	xfree((char *)e, sizeof(Exp));
+}
+
+void free_E_list(E_list * l)
+{
+	if (l == NULL) return;
+	free_E_list(l->next);
+	free_Exp(l->e);
+	xfree((char *)l, sizeof(E_list));
+}
+
+/* Returns the number of connectors in the expression e */
+int size_of_expression(Exp * e)
+{
+	int size;
+	E_list * l;
+	if (e->type == CONNECTOR_type) return 1;
+	size = 0;
+	for (l=e->u.l; l!=NULL; l=l->next) {
+		size += size_of_expression(l->e);
+	}
+	return size;
+}
+
+/** 
+ * Build a copy of the given expression (don't copy strings, of course)
+ */
+static E_list * copy_E_list(E_list * l);
+Exp * copy_Exp(Exp * e)
+{
+	Exp * n;
+	if (e == NULL) return NULL;
+	n = (Exp *) xalloc(sizeof(Exp));
+	*n = *e;
+	if (e->type != CONNECTOR_type) {
+		n->u.l = copy_E_list(e->u.l);
+	}
+	return n;
+}
+
+static E_list * copy_E_list(E_list * l)
+{
+	E_list * nl;
+	if (l == NULL) return NULL;
+	nl = (E_list *) xalloc(sizeof(E_list));
+	nl->next = copy_E_list(l->next);
+	nl->e = copy_Exp(l->e);
+	return nl;
+}
+
+/**
+ * Compare two expressions, return 1 for equal, 0 for unequal
+ */
+static int exp_compare(Exp * e1, Exp * e2)
+{
+	E_list *el1, *el2;
+
+	if ((e1 == NULL) && (e2 == NULL))
+	  return 1; /* they are equal */
+	if ((e1 == NULL) || (e2 == NULL))
+	  return 0; /* they are not equal */
+	if (e1->type != e2->type)
+		return 0;
+	if (fabs (e1->cost - e2->cost) > 0.001)
+		return 0;
+	if (e1->type == CONNECTOR_type)
+	{
+		if (e1->dir != e2->dir)
+			return 0;
+		/* printf("%s %s\n",e1->u.string,e2->u.string); */
+		if (strcmp(e1->u.string,e2->u.string)!=0)
+			return 0;
+	}
+	else
+	{
+		el1 = e1->u.l;
+		el2 = e2->u.l;
+		/* while at least 1 is non-null */
+		for (;(el1!=NULL)||(el2!=NULL);) {
+		  /*fail if 1 is null */
+			if ((el1==NULL)||(el2==NULL))
+				return 0;
+			/* fail if they are not compared */
+			if (exp_compare(el1->e, el2->e) == 0)
+				return 0;
+			if (el1!=NULL)
+				el1 = el1->next;
+			if (el2!=NULL)
+				el2 = el2->next;
+		}
+	}
+	return 1; /* if never returned 0, return 1 */
+}
+
+/**
+ * Sub-expression matcher -- return 1 if sub is non-NULL and 
+ * contained in super, 0 otherwise.
+ */
+static int exp_contains(Exp * super, Exp * sub)
+{
+	E_list * el;
+
+#ifdef DEBUG
+	printf("SUP: ");
+	if (super) print_expression(super); 
+	printf("\n");
+#endif
+
+	if (sub==NULL || super==NULL)
+		return 0;
+	if (exp_compare(sub,super)==1)
+		return 1;
+	if (super->type==CONNECTOR_type)
+	  return 0; /* super is a leaf */
+
+	/* proceed through supers children and return 1 if sub
+	   is contained in any of them */
+	for(el = super->u.l; el!=NULL; el=el->next) {
+		if (exp_contains(el->e, sub)==1)
+			return 1;
+	}
+	return 0;
+}
+
+/* ======================================================== */
+/* X_node utilities ... */
+/** 
+ * frees the list of X_nodes pointed to by x, and all of the expressions
+ */
+void free_X_nodes(X_node * x)
+{
+	X_node * y;
+	for (; x!= NULL; x = y) {
+		y = x->next;
+		free_Exp(x->exp);
+		xfree((char *)x, sizeof(X_node));
+	}
+}
+
+/**
+ * Destructively catenates the two disjunct lists d1 followed by d2.
+ * Doesn't change the contents of the disjuncts.
+ * Traverses the first list, but not the second.
+ */
+X_node * catenate_X_nodes(X_node *d1, X_node *d2)
+{
+	X_node * dis = d1;
+
+	if (d1 == NULL) return d2;
+	if (d2 == NULL) return d1;
+	while (dis->next != NULL) dis = dis->next;
+	dis->next = d2;
+	return d1;
+}
+
+/* ======================================================== */
+/* Connector utilities ... */
+
 /**
  * This hash function that takes a connector and a seed value i.
  * It only looks at the leading upper case letters of
@@ -55,18 +225,13 @@ void free_connectors(Connector *e)
 	}
 }
 
-/**
- * free_disjuncts() -- free the list of disjuncts pointed to by c
- * (does not free any strings)
- */
-void free_disjuncts(Disjunct *c)
+void exfree_connectors(Connector *e)
 {
-	Disjunct *c1;
-	for (;c != NULL; c = c1) {
-		c1 = c->next;
-		free_connectors(c->left);
-		free_connectors(c->right);
-		xfree((char *)c, sizeof(Disjunct));
+	Connector * n;
+	for(;e != NULL; e = n) {
+		n = e->next;
+		exfree((void *) e->string, sizeof(char)*(strlen(e->string)+1));
+		exfree(e, sizeof(Connector));
 	}
 }
 
@@ -91,74 +256,6 @@ Connector * init_connector(Connector *c)
 	return c;
 }
 
-/** 
- * frees the list of X_nodes pointed to by x, and all of the expressions
- */
-void free_X_nodes(X_node * x)
-{
-	X_node * y;
-	for (; x!= NULL; x = y) {
-		y = x->next;
-		free_Exp(x->exp);
-		xfree((char *)x, sizeof(X_node));
-	}
-}
-
-void free_E_list(E_list *);
-void free_Exp(Exp * e)
-{
-	if (e->type != CONNECTOR_type) {
-		free_E_list(e->u.l);
-	}
-	xfree((char *)e, sizeof(Exp));
-}
-
-void free_E_list(E_list * l)
-{
-	if (l == NULL) return;
-	free_E_list(l->next);
-	free_Exp(l->e);
-	xfree((char *)l, sizeof(E_list));
-}
-
-/* Returns the number of connectors in the expression e */
-int size_of_expression(Exp * e)
-{
-	int size;
-	E_list * l;
-	if (e->type == CONNECTOR_type) return 1;
-	size = 0;
-	for (l=e->u.l; l!=NULL; l=l->next) {
-		size += size_of_expression(l->e);
-	}
-	return size;
-}
-
-/* Build a copy of the given expression (don't copy strings, of course) */
-static E_list * copy_E_list(E_list * l);
-Exp * copy_Exp(Exp * e)
-{
-	Exp * n;
-	if (e == NULL) return NULL;
-	n = (Exp *) xalloc(sizeof(Exp));
-	*n = *e;
-	if (e->type != CONNECTOR_type) {
-		n->u.l = copy_E_list(e->u.l);
-	}
-	return n;
-}
-
-static E_list * copy_E_list(E_list * l)
-{
-	E_list * nl;
-	if (l == NULL) return NULL;
-	nl = (E_list *) xalloc(sizeof(E_list));
-	*nl = *l;	/* not necessary -- both fields will be built below */
-	nl->next = copy_E_list(l->next);
-	nl->e = copy_Exp(l->e);
-	return nl;
-}
-
 /**
  * This builds a new copy of the connector list pointed to by c.
  * Strings, as usual, are not copied.
@@ -173,33 +270,6 @@ Connector * copy_connectors(Connector * c)
 	return c1;
 }
 
-/**
- * This builds a new copy of the disjunct pointed to by d (except for the
- * next field which is set to NULL).  Strings, as usual,
- * are not copied.
- */
-Disjunct * copy_disjunct(Disjunct * d)
-{
-	Disjunct * d1;
-	if (d == NULL) return NULL;
-	d1 = (Disjunct *) xalloc(sizeof(Disjunct));
-	*d1 = *d;
-	d1->next = NULL;
-	d1->left = copy_connectors(d->left);
-	d1->right = copy_connectors(d->right);
-	return d1;
-}
-
-void exfree_connectors(Connector *e)
-{
-	Connector * n;
-	for(;e != NULL; e = n) {
-		n = e->next;
-		exfree((void *) e->string, sizeof(char)*(strlen(e->string)+1));
-		exfree(e, sizeof(Connector));
-	}
-}
-
 Connector * excopy_connectors(Connector * c)
 {
 	char * s;
@@ -217,6 +287,9 @@ Connector * excopy_connectors(Connector * c)
 	return c1;
 }
 
+/* ======================================================== */
+/* Link utilities ... */
+
 Link * excopy_link(Link * l)
 {
 	char * s;
@@ -244,86 +317,8 @@ void exfree_link(Link * l)
 	exfree(l, sizeof(Link));
 }
 
-
-/**
- * Destructively catenates the two disjunct lists d1 followed by d2.
- * Doesn't change the contents of the disjuncts.
- * Traverses the first list, but not the second.
- */
-Disjunct * catenate_disjuncts(Disjunct *d1, Disjunct *d2)
-{
-	Disjunct * dis = d1;
-
-	if (d1 == NULL) return d2;
-	if (d2 == NULL) return d1;
-	while (dis->next != NULL) dis = dis->next;
-	dis->next = d2;
-	return d1;
-}
-
-/**
- * Destructively catenates the two disjunct lists d1 followed by d2.
- * Doesn't change the contents of the disjuncts.
- * Traverses the first list, but not the second.
- */
-X_node * catenate_X_nodes(X_node *d1, X_node *d2)
-{
-	X_node * dis = d1;
-
-	if (d1 == NULL) return d2;
-	if (d2 == NULL) return d1;
-	while (dis->next != NULL) dis = dis->next;
-	dis->next = d2;
-	return d1;
-}
-
-/** Returns TRUE if one of the words in the sentence is s */
-int sentence_contains(Sentence sent, const char * s)
-{
-	int w;
-	for (w=0; w<sent->length; w++) {
-		if (strcmp(sent->word[w].string, s) == 0) return TRUE;
-	}
-	return FALSE;
-}
-
-void set_is_conjunction(Sentence sent)
-{
-	int w;
-	char * s;
-	for (w=0; w<sent->length; w++) {
-		s = sent->word[w].string;
-		sent->is_conjunction[w] = ((strcmp(s, "and")==0) || (strcmp(s, "or" )==0) ||
-		                           (strcmp(s, "but")==0) || (strcmp(s, "nor")==0));
-	}
-}
-
-/**
- * Return true if the sentence contains a conjunction.  Assumes
- * is_conjunction[] has been initialized.
- */
-int sentence_contains_conjunction(Sentence sent)
-{
-	int w;
-	if (NULL == sent->is_conjunction) return FALSE;
-
-	for (w=0; w<sent->length; w++) {
-		if (sent->is_conjunction[w]) return TRUE;
-	}
-	return FALSE;
-}
-
-/**
- *  Returns true if the range lw...rw inclusive contains a conjunction 
- */
-int conj_in_range(Sentence sent, int lw, int rw)
-{
-	for (;lw < rw+1; lw++) {
-		if (sent->is_conjunction[lw]) return TRUE;
-	}
-	return FALSE;
-}
-
+/* ======================================================== */
+/* Connector-set utilities ... */
 /**
  * This hash function only looks at the leading upper case letters of
  * the string, and the direction, '+' or '-'.
@@ -398,16 +393,8 @@ int match_in_connector_set(Sentence sent, Connector_set *conset, Connector * c,
 	return FALSE;
 }
 
-Dict_node * list_whole_dictionary(Dict_node *root, Dict_node *dn)
-{
-	Dict_node *c, *d;
-	if (root == NULL) return dn;
-	c = (Dict_node *) xalloc(sizeof(Dict_node));
-	*c = *root;
-	d = list_whole_dictionary(root->left, dn);
-	c->right = list_whole_dictionary(root->right, d);
-	return c;
-}
+/* ======================================================== */
+/* More connector utilities ... */
 
 /**
  * This is like the basic "match" function in count.c - the basic
@@ -464,71 +451,8 @@ int word_has_connector(Dict_node * dn, const char * cs, int direction)
 	return 0;
 }
 
-/* ============================================================================= */
-
-/* 1 for equal, 0 for unequal */
-static int exp_compare(Exp * e1, Exp * e2)
-{
-	E_list *el1, *el2;
-
-	if ((e1 == NULL) && (e2 == NULL))
-	  return 1; /* they are equal */
-	if ((e1 == NULL) || (e2 == NULL))
-	  return 0; /* they are not equal */
-	if (e1->type != e2->type)
-		return 0;
-	if (e1->cost != e2->cost)
-		return 0;
-	if (e1->type == CONNECTOR_type) {
-		if (e1->dir != e2->dir)
-			return 0;
-		/* printf("%s %s\n",e1->u.string,e2->u.string); */
-		if (strcmp(e1->u.string,e2->u.string)!=0)
-			return 0;
-	} else {
-		el1 = e1->u.l;
-		el2 = e2->u.l;
-		/* while at least 1 is non-null */
-		for (;(el1!=NULL)||(el2!=NULL);) {
-		  /*fail if 1 is null */
-			if ((el1==NULL)||(el2==NULL))
-				return 0;
-			/* fail if they are not compared */
-			if (exp_compare(el1->e, el2->e) == 0)
-				return 0;
-			if (el1!=NULL)
-				el1 = el1->next;
-			if (el2!=NULL)
-				el2 = el2->next;
-		}
-	}
-	return 1; /* if never returned 0, return 1 */
-}
-
-/* 1 if sub is non-NULL and contained in super, 0 otherwise */
-static int exp_contains(Exp * super, Exp * sub)
-{
-	E_list * el;
-
-	/* printf("\nSUP:");
-	   if (super!=NULL)
-	   print_expression(super); */
-
-	if (sub==NULL || super==NULL)
-		return 0;
-	if (exp_compare(sub,super)==1)
-		return 1;
-	if (super->type==CONNECTOR_type)
-	  return 0; /* super is a leaf */
-
-	/* proceed through supers children and return 1 if sub
-	   is contained in any of them */
-	for(el = super->u.l; el!=NULL; el=el->next) {
-		if (exp_contains(el->e, sub)==1)
-			return 1;
-	}
-	return 0;
-}
+/* ======================================================== */
+/* Dictionary utilities ... */
 
 static int dn_word_contains(Dictionary dict,
                             Dict_node * w_dn, const char * macro)
@@ -544,10 +468,13 @@ static int dn_word_contains(Dictionary dict,
 	m_exp = m_dn->exp;
 	free_lookup_list(m_dn);
 
-	/* printf("\n\nWORD:");
-	 print_expression(w_dn->exp);
-	 printf("\nMACR:\n");
-	 print_expression(m_exp); */
+#ifdef DEBUG
+	printf("\nWORD: ");
+	print_expression(w_dn->exp);
+	printf("\nMACR: ");
+	print_expression(m_exp); 
+	printf("\n");
+#endif
 
 	for (;w_dn != NULL; w_dn = w_dn->right)
 	{
@@ -574,10 +501,22 @@ int word_contains(Dictionary dict, const char * word, const char * macro)
 	return ret;
 }
 
+Dict_node * list_whole_dictionary(Dict_node *root, Dict_node *dn)
+{
+	Dict_node *c, *d;
+	if (root == NULL) return dn;
+	c = (Dict_node *) xalloc(sizeof(Dict_node));
+	*c = *root;
+	d = list_whole_dictionary(root->left, dn);
+	c->right = list_whole_dictionary(root->right, d);
+	return c;
+}
+
 #define PAST_TENSE_FORM_MARKER "<marker-past>"
 #define ENTITY_MARKER          "<marker-entity>"
 #define COMMON_ENTITY_MARKER   "<marker-common-entity>"
 
+/* This is exported to public API (for Java) */
 int dictionary_is_past_tense_form(Dictionary dict, const char * str)
 {
 	if (word_contains(dict, str, PAST_TENSE_FORM_MARKER) == 1)
@@ -591,6 +530,7 @@ int dictionary_is_past_tense_form(Dictionary dict, const char * str)
  * names of people), street addresses, phone numbers,
  * etc.
  */
+/* This is exported to public API (for Java) */
 int dictionary_is_entity(Dictionary dict, const char * str)
 {
 	if (word_contains(dict, str, ENTITY_MARKER) == 1)
@@ -598,16 +538,4 @@ int dictionary_is_entity(Dictionary dict, const char * str)
 	return 0;
 }
 
-/**
- * dictionary_is_common_entity - Return true if word is a common
- * noun or adjective, typically appearing in corporate entity names.
- * e.g. "Sun State Bank" -- sun, state and bank are all common nouns.
- */
-int dictionary_is_common_entity(Dictionary dict, const char * str)
-{
-	if (word_contains(dict, str, COMMON_ENTITY_MARKER) == 1)
-		return 1;
-	return 0;
-}
-
 /* ========================= END OF FILE ============================== */
